@@ -41,6 +41,7 @@
 #include "isimodem.h"
 #include "isiutil.h"
 #include "ss.h"
+#include "debug.h"
 
 struct settings_data {
 	GIsiClient *client;
@@ -88,8 +89,9 @@ static void update_status_mask(unsigned int *mask, int bsc)
 	}
 }
 
-static bool query_resp_cb(GIsiClient *client, const void *restrict data,
-				size_t len, uint16_t object, void *opaque)
+static gboolean query_resp_cb(GIsiClient *client,
+				const void *restrict data, size_t len,
+				uint16_t object, void *opaque)
 {
 	GIsiSubBlockIter iter;
 	const unsigned char *msg = data;
@@ -132,13 +134,13 @@ static bool query_resp_cb(GIsiClient *client, const void *restrict data,
 			for (i = 0; i < count; i++) {
 				if (!g_isi_sb_iter_get_byte(&iter, &bsc, 3 + i))
 					goto error;
-			        update_status_mask(&mask, bsc);
+				update_status_mask(&mask, bsc);
 			}
 			break;
 		}
 		default:
-			DBG("Skipping sub-block: 0x%04X (%zu bytes)",
-				g_isi_sb_iter_get_id(&iter),
+			DBG("Skipping sub-block: %s (%zd bytes)",
+				ss_subblock_name(g_isi_sb_iter_get_id(&iter)),
 				g_isi_sb_iter_get_len(&iter));
 			break;
 		}
@@ -153,7 +155,7 @@ error:
 
 out:
 	g_free(cbd);
-	return true;
+	return TRUE;
 
 }
 
@@ -187,8 +189,9 @@ error:
 	g_free(cbd);
 }
 
-static bool set_resp_cb(GIsiClient *client, const void *restrict data,
-				size_t len, uint16_t object, void *opaque)
+static gboolean set_resp_cb(GIsiClient *client,
+				const void *restrict data, size_t len,
+				uint16_t object, void *opaque)
 {
 	GIsiSubBlockIter iter;
 	const unsigned char *msg = data;
@@ -228,8 +231,8 @@ static bool set_resp_cb(GIsiClient *client, const void *restrict data,
 			break;
 		}
 		default:
-			DBG("Skipping sub-block: 0x%04X (%zu bytes)",
-				g_isi_sb_iter_get_id(&iter),
+			DBG("Skipping sub-block: %s (%zd bytes)",
+				ss_subblock_name(g_isi_sb_iter_get_id(&iter)),
 				g_isi_sb_iter_get_len(&iter));
 			break;
 		}
@@ -243,7 +246,7 @@ error:
 
 out:
 	g_free(cbd);
-	return true;
+	return TRUE;
 
 }
 
@@ -286,25 +289,32 @@ static gboolean isi_call_settings_register(gpointer user)
 	return FALSE;
 }
 
-static void reachable_cb(GIsiClient *client, bool alive, uint16_t object,
+static void reachable_cb(GIsiClient *client, gboolean alive, uint16_t object,
 				void *opaque)
 {
 	struct ofono_call_settings *cs = opaque;
+	const char *debug = NULL;
 
-	if (alive == true) {
-		DBG("Resource 0x%02X, with version %03d.%03d reachable",
-			g_isi_client_resource(client),
-			g_isi_version_major(client),
-			g_isi_version_minor(client));
-		g_idle_add(isi_call_settings_register, cs);
+	if (!alive) {
+		DBG("Unable to bootstrap call settings driver");
 		return;
 	}
-	DBG("Unable to bootsrap call settings driver");
+
+	DBG("%s (v%03d.%03d) reachable",
+		pn_resource_name(g_isi_client_resource(client)),
+		g_isi_version_major(client),
+		g_isi_version_minor(client));
+
+	debug = getenv("OFONO_ISI_DEBUG");
+	if (debug && (strcmp(debug, "all") == 0 || strcmp(debug, "ss") == 0))
+		g_isi_client_set_debug(client, ss_debug, NULL);
+
+	g_idle_add(isi_call_settings_register, cs);
 }
 
 
-static int isi_call_settings_probe(struct ofono_call_settings *cs, unsigned int vendor,
-					void *user)
+static int isi_call_settings_probe(struct ofono_call_settings *cs,
+					unsigned int vendor, void *user)
 {
 	GIsiModem *idx = user;
 	struct settings_data *data;
@@ -320,6 +330,7 @@ static int isi_call_settings_probe(struct ofono_call_settings *cs, unsigned int
 		return -ENOMEM;
 
 	ofono_call_settings_set_data(cs, data);
+
 	if (!g_isi_verify(data->client, reachable_cb, cs))
 		DBG("Unable to verify reachability");
 
@@ -330,10 +341,12 @@ static void isi_call_settings_remove(struct ofono_call_settings *cs)
 {
 	struct settings_data *data = ofono_call_settings_get_data(cs);
 
-	if (data) {
-		g_isi_client_destroy(data->client);
-		g_free(data);
-	}
+	if (!data)
+		return;
+
+	ofono_call_settings_set_data(cs, NULL);
+	g_isi_client_destroy(data->client);
+	g_free(data);
 }
 
 static struct ofono_call_settings_driver driver = {
