@@ -52,8 +52,8 @@ static void mbm_stke_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	ofono_stk_envelope_cb_t cb = cbd->cb;
 	GAtResultIter iter;
 	struct ofono_error error;
-	const guint8 *pdu;
-	gint len;
+	const guint8 *pdu = NULL;
+	gint len = 0;
 
 	decode_at_error(&error, g_at_result_final_response(result));
 
@@ -64,11 +64,9 @@ static void mbm_stke_cb(gboolean ok, GAtResult *result, gpointer user_data)
 
 	g_at_result_iter_init(&iter, result);
 
-	if (g_at_result_iter_next(&iter, "*STKE:") == FALSE)
-		goto error;
-
-	if (g_at_result_iter_next_hexstring(&iter, &pdu, &len) == FALSE)
-		goto error;
+	if (g_at_result_iter_next(&iter, "*STKE:") == TRUE)
+		if (g_at_result_iter_next_hexstring(&iter, &pdu, &len) == FALSE)
+			goto error;
 
 	cb(&error, pdu, len, cbd->data);
 	return;
@@ -181,29 +179,25 @@ static void stkn_notify(GAtResult *result, gpointer user_data)
 
 static void stkend_notify(GAtResult *result, gpointer user_data)
 {
+	struct ofono_stk *stk = user_data;
+
+	ofono_stk_proactive_session_end_notify(stk);
 }
 
 static void mbm_stkc_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
-}
-
-static gboolean mbm_stk_register(gpointer user)
-{
-	struct ofono_stk *stk = user;
+	struct ofono_stk *stk = user_data;
 	struct stk_data *sd = ofono_stk_get_data(stk);
 
+	if (!ok)
+		return;
+
 	g_at_chat_register(sd->chat, "*STKI:", stki_notify, FALSE, stk, NULL);
 	g_at_chat_register(sd->chat, "*STKN:", stkn_notify, FALSE, stk, NULL);
 	g_at_chat_register(sd->chat, "*STKEND",
 					stkend_notify, FALSE, stk, NULL);
 
-	/* Perform PROFILE DOWNLOAD and enable *STKI / *STKN */
-	g_at_chat_send(sd->chat, "AT*STKC=1,\"19E1FFFF0000FF7FFF03FEFF\"",
-			none_prefix, mbm_stkc_cb, stk, NULL);
-
 	ofono_stk_register(stk);
-
-	return FALSE;
 }
 
 static int mbm_stk_probe(struct ofono_stk *stk, unsigned int vendor, void *data)
@@ -215,7 +209,10 @@ static int mbm_stk_probe(struct ofono_stk *stk, unsigned int vendor, void *data)
 	sd->chat = chat;
 
 	ofono_stk_set_data(stk, sd);
-	g_idle_add(mbm_stk_register, stk);
+
+	/* Perform PROFILE DOWNLOAD and enable *STKI / *STKN */
+	g_at_chat_send(sd->chat, "AT*STKC=1,\"19E1FFFF0000FF7FFF03FEFF\"",
+			none_prefix, mbm_stkc_cb, stk, NULL);
 
 	return 0;
 }
